@@ -24,6 +24,7 @@
 # - Tomas Javurek <tomas.javurek@cern.ch>, 2020
 # - Martin Barisits <martin.barisits@cern.ch>, 2020
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
+# - Thomas Beermann <thomas.beermann@cern.ch>, 2021
 
 import os
 import logging
@@ -36,17 +37,13 @@ from rucio.common.utils import execute, PREFERRED_CHECKSUM
 class Default(protocol.RSEProtocol):
     """ Implementing access to RSEs using the XRootD protocol using GSI authentication."""
 
-    def __init__(self, protocol_attr, rse_settings, logger=None):
+    def __init__(self, protocol_attr, rse_settings, logger=logging.log):
         """ Initializes the object with information about the referred RSE.
 
             :param props Properties derived from the RSE Repository
         """
         super(Default, self).__init__(protocol_attr, rse_settings, logger=logger)
 
-        if not logger:
-            logger = logging.getLogger('%s.null' % __name__)
-            logger.disabled = True
-
         self.scheme = self.attributes['scheme']
         self.hostname = self.attributes['hostname']
         self.port = str(self.attributes['port'])
@@ -61,7 +58,7 @@ class Default(protocol.RSEProtocol):
             :returns: Fully qualified PFN.
 
         """
-        self.logger.debug('xrootd.path2pfn: path: {}'.format(path))
+        self.logger(logging.DEBUG, 'xrootd.path2pfn: path: {}'.format(path))
         if not path.startswith('xroot') and not path.startswith('root'):
             if path.startswith('/'):
                 return '%s://%s:%s/%s' % (self.scheme, self.hostname, self.port, path)
@@ -79,11 +76,11 @@ class Default(protocol.RSEProtocol):
 
             :raise  ServiceUnavailable
         """
-        self.logger.debug('xrootd.exists: pfn: {}'.format(pfn))
+        self.logger(logging.DEBUG, 'xrootd.exists: pfn: {}'.format(pfn))
         try:
             path = self.pfn2path(pfn)
             cmd = 'XrdSecPROTOCOL=gsi xrdfs %s:%s stat %s' % (self.hostname, self.port, path)
-            self.logger.info('xrootd.exists: cmd: {}'.format(cmd))
+            self.logger(logging.INFO, 'xrootd.exists: cmd: {}'.format(cmd))
             status, out, err = execute(cmd)
             if status != 0:
                 return False
@@ -102,7 +99,7 @@ class Default(protocol.RSEProtocol):
 
         :returns: a dict with two keys, filesize and an element of GLOBALLY_SUPPORTED_CHECKSUMS.
         """
-        self.logger.debug('xrootd.stat: path: {}'.format(path))
+        self.logger(logging.DEBUG, 'xrootd.stat: path: {}'.format(path))
         ret = {}
         chsum = None
         if path.startswith('root:'):
@@ -111,14 +108,14 @@ class Default(protocol.RSEProtocol):
         try:
             # xrdfs stat for getting filesize
             cmd = 'XrdSecPROTOCOL=gsi xrdfs %s:%s stat %s' % (self.hostname, self.port, path)
-            self.logger.info('xrootd.stat: filesize cmd: {}'.format(cmd))
+            self.logger(logging.INFO, 'xrootd.stat: filesize cmd: {}'.format(cmd))
             status_stat, out, err = execute(cmd)
             if status_stat == 0:
                 ret['filesize'] = out.split(b'\n')[2].split()[-1].decode()
 
             # xrdfs query checksum for getting checksum
             cmd = 'XrdSecPROTOCOL=gsi xrdfs %s:%s query checksum %s' % (self.hostname, self.port, path)
-            self.logger.info('xrootd.stat: checksum cmd: {}'.format(cmd))
+            self.logger(logging.INFO, 'xrootd.stat: checksum cmd: {}'.format(cmd))
             status_query, out, err = execute(cmd)
             if status_query == 0:
                 chsum, value = out.strip(b'\n').split()
@@ -145,7 +142,7 @@ class Default(protocol.RSEProtocol):
 
         :returns: path.
         """
-        self.logger.debug('xrootd.pfn2path: pfn: {}'.format(pfn))
+        self.logger(logging.DEBUG, 'xrootd.pfn2path: pfn: {}'.format(pfn))
         if pfn.startswith('//'):
             return pfn
         elif pfn.startswith('/'):
@@ -164,7 +161,7 @@ class Default(protocol.RSEProtocol):
 
         :returns: Fully qualified PFN.
         """
-        self.logger.debug('xrootd.lfns2pfns: lfns: {}'.format(lfns))
+        self.logger(logging.DEBUG, 'xrootd.lfns2pfns: lfns: {}'.format(lfns))
         pfns = {}
         prefix = self.attributes['prefix']
 
@@ -192,12 +189,12 @@ class Default(protocol.RSEProtocol):
 
             :raises RSEAccessDenied
         """
-        self.logger.debug('xrootd.connect: port: {}, hostname {}'.format(self.port, self.hostname))
+        self.logger(logging.DEBUG, 'xrootd.connect: port: {}, hostname {}'.format(self.port, self.hostname))
         try:
             # The query stats call is not implemented on some xroot doors.
             # Workaround: fail, if server does not reply within 10 seconds for static config query
             cmd = 'XrdSecPROTOCOL=gsi XRD_REQUESTTIMEOUT=10 xrdfs %s:%s query config %s:%s' % (self.hostname, self.port, self.hostname, self.port)
-            self.logger.info('xrootd.connect: cmd: {}'.format(cmd))
+            self.logger(logging.INFO, 'xrootd.connect: cmd: {}'.format(cmd))
             status, out, err = execute(cmd)
             if status != 0:
                 raise exception.RSEAccessDenied(err)
@@ -217,10 +214,10 @@ class Default(protocol.RSEProtocol):
 
             :raises DestinationNotAccessible, ServiceUnavailable, SourceNotFound
         """
-        self.logger.debug('xrootd.get: pfn: {}'.format(pfn))
+        self.logger(logging.DEBUG, 'xrootd.get: pfn: {}'.format(pfn))
         try:
             cmd = 'XrdSecPROTOCOL=gsi xrdcp -f %s %s' % (pfn, dest)
-            self.logger.info('xrootd.get: cmd: {}'.format(cmd))
+            self.logger(logging.INFO, 'xrootd.get: cmd: {}'.format(cmd))
             status, out, err = execute(cmd)
             if status == 54:
                 raise exception.SourceNotFound()
@@ -242,16 +239,16 @@ class Default(protocol.RSEProtocol):
             :raises ServiceUnavailable: if some generic error occured in the library.
             :raises SourceNotFound: if the source file was not found on the referred storage.
         """
-        self.logger.debug('xrootd.put: filename: {} target: {}'.format(filename, target))
+        self.logger(logging.DEBUG, 'xrootd.put: filename: {} target: {}'.format(filename, target))
         source_dir = source_dir or '.'
         source_url = '%s/%s' % (source_dir, filename)
-        self.logger.debug('xrootd put: source url: {}'.format(source_url))
+        self.logger(logging.DEBUG, 'xrootd put: source url: {}'.format(source_url))
         path = self.path2pfn(target)
         if not os.path.exists(source_url):
             raise exception.SourceNotFound()
         try:
             cmd = 'XrdSecPROTOCOL=gsi xrdcp -f %s %s' % (source_url, path)
-            self.logger.info('xrootd.put: cmd: {}'.format(cmd))
+            self.logger(logging.INFO, 'xrootd.put: cmd: {}'.format(cmd))
             status, out, err = execute(cmd)
             if status != 0:
                 raise exception.RucioException(err)
@@ -267,13 +264,13 @@ class Default(protocol.RSEProtocol):
             :raises ServiceUnavailable: if some generic error occured in the library.
             :raises SourceNotFound: if the source file was not found on the referred storage.
         """
-        self.logger.debug('xrootd.delete: pfn: {}'.format(pfn))
+        self.logger(logging.DEBUG, 'xrootd.delete: pfn: {}'.format(pfn))
         if not self.exists(pfn):
             raise exception.SourceNotFound()
         try:
             path = self.pfn2path(pfn)
             cmd = 'XrdSecPROTOCOL=gsi xrdfs %s:%s rm %s' % (self.hostname, self.port, path)
-            self.logger.info('xrootd.delete: cmd: {}'.format(cmd))
+            self.logger(logging.INFO, 'xrootd.delete: cmd: {}'.format(cmd))
             status, out, err = execute(cmd)
             if status != 0:
                 raise exception.RucioException(err)
@@ -289,7 +286,7 @@ class Default(protocol.RSEProtocol):
             :raises ServiceUnavailable: if some generic error occured in the library.
             :raises SourceNotFound: if the source file was not found on the referred storage.
         """
-        self.logger.debug('xrootd.rename: pfn: {}'.format(pfn))
+        self.logger(logging.DEBUG, 'xrootd.rename: pfn: {}'.format(pfn))
         if not self.exists(pfn):
             raise exception.SourceNotFound()
         try:
@@ -297,10 +294,10 @@ class Default(protocol.RSEProtocol):
             new_path = self.pfn2path(new_pfn)
             new_dir = new_path[:new_path.rindex('/') + 1]
             cmd = 'XrdSecPROTOCOL=gsi xrdfs %s:%s mkdir -p %s' % (self.hostname, self.port, new_dir)
-            self.logger.info('xrootd.stat: mkdir cmd: {}'.format(cmd))
+            self.logger(logging.INFO, 'xrootd.stat: mkdir cmd: {}'.format(cmd))
             status, out, err = execute(cmd)
             cmd = 'XrdSecPROTOCOL=gsi xrdfs %s:%s mv %s %s' % (self.hostname, self.port, path, new_path)
-            self.logger.info('xrootd.stat: rename cmd: {}'.format(cmd))
+            self.logger(logging.INFO, 'xrootd.stat: rename cmd: {}'.format(cmd))
             status, out, err = execute(cmd)
             if status != 0:
                 raise exception.RucioException(err)
