@@ -55,10 +55,9 @@ except ImportError:
 from rucio.common import exception, utils, constants
 from rucio.common.config import config_get_int
 from rucio.common.constraints import STRING_TYPES
+from rucio.common.logging import formatted_logger
 from rucio.common.utils import make_valid_did, GLOBALLY_SUPPORTED_CHECKSUMS
 
-_logger = logging.getLogger(__name__)
-
 
 def get_rse_info(rse=None, vo='def', rse_id=None, session=None):
     """
@@ -170,16 +169,17 @@ def select_protocol(rse_settings, operation, scheme=None, domain='wan'):
     return min(candidates, key=lambda k: k['domains'][domain][operation])
 
 
-def create_protocol(rse_settings, operation, scheme=None, domain='wan', auth_token=None, logger=_logger):
+def create_protocol(rse_settings, operation, scheme=None, domain='wan', auth_token=None, logger=logging.log):
     """
     Instanciates the protocol defined for the given operation.
 
     :param rse_settings:  RSE attributes
-    :param operation: Intended operation for this protocol
-    :param scheme:    Optional filter if no specific protocol is defined in rse_setting for the provided operation
-    :param domain:    Optional specification of the domain
-    :param auth_token: Optionally passing JSON Web Token (OIDC) string for authentication
-    :returns:         An instance of the requested protocol
+    :param operation:     Intended operation for this protocol
+    :param scheme:        Optional filter if no specific protocol is defined in rse_setting for the provided operation
+    :param domain:        Optional specification of the domain
+    :param auth_token:    Optionally passing JSON Web Token (OIDC) string for authentication
+    :param logger:        Optional decorated logger that can be passed from the calling daemons or servers.
+    :returns:             An instance of the requested protocol
     """
 
     # Verify feasibility of Protocol
@@ -194,33 +194,36 @@ def create_protocol(rse_settings, operation, scheme=None, domain='wan', auth_tok
 
     # Instantiate protocol
     comp = protocol_attr['impl'].split('.')
+    prefix = '.'.join(comp[-2:]) + ': '
+    logger = formatted_logger(logger, prefix + "%s")
     mod = __import__('.'.join(comp[:-1]))
     for n in comp[1:]:
         try:
             mod = getattr(mod, n)
         except AttributeError as e:
-            logger.debug('Protocol implementations not supported.')
+            logger(logging.DEBUG, 'Protocol implementations not supported.')
             raise exception.RucioException(str(e))  # TODO: provide proper rucio exception
     protocol_attr['auth_token'] = auth_token
     protocol = mod(protocol_attr, rse_settings, logger=logger)
     return protocol
 
 
-def lfns2pfns(rse_settings, lfns, operation='write', scheme=None, domain='wan', auth_token=None):
+def lfns2pfns(rse_settings, lfns, operation='write', scheme=None, domain='wan', auth_token=None, logger=logging.log):
     """
         Convert the lfn to a pfn
 
-        :rse_settings:   RSE attributes
+        :rse_settings:      RSE attributes
         :param lfns:        logical file names as a dict containing 'scope' and 'name' as keys. For bulk a list of dicts can be provided
-        :param operation: Intended operation for this protocol
-        :param scheme:    Optional filter if no specific protocol is defined in rse_setting for the provided operation
-        :param domain:    Optional specification of the domain
-        :param auth_token: Optionally passing JSON Web Token (OIDC) string for authentication
+        :param operation:   Intended operation for this protocol
+        :param scheme:      Optional filter if no specific protocol is defined in rse_setting for the provided operation
+        :param domain:      Optional specification of the domain
+        :param auth_token:  Optionally passing JSON Web Token (OIDC) string for authentication
+        :param logger:      Optional decorated logger that can be passed from the calling daemons or servers.
 
-        :returns: a dict with scope:name as key and the PFN as value
+        :returns:           a dict with scope:name as key and the PFN as value
 
     """
-    return create_protocol(rse_settings, operation, scheme, domain, auth_token=auth_token).lfns2pfns(lfns)
+    return create_protocol(rse_settings, operation, scheme, domain, auth_token=auth_token, logger=logger).lfns2pfns(lfns)
 
 
 def parse_pfns(rse_settings, pfns, operation='read', domain='wan', auth_token=None):
@@ -245,20 +248,20 @@ def parse_pfns(rse_settings, pfns, operation='read', domain='wan', auth_token=No
     return create_protocol(rse_settings, operation, urlparse(pfns[0]).scheme, domain, auth_token=auth_token).parse_pfns(pfns)
 
 
-def exists(rse_settings, files, domain='wan', auth_token=None, logger=_logger):
+def exists(rse_settings, files, domain='wan', auth_token=None, logger=logging.log):
     """
         Checks if a file is present at the connected storage.
         Providing a list indicates the bulk mode.
 
-        :rse_settings:   RSE attributes
-        :param files: a single dict or a list with dicts containing 'scope' and 'name'
-                      if LFNs are used and only 'name' if PFNs are used.
-                      E.g. {'name': '2_rse_remote_get.raw', 'scope': 'user.jdoe'}, {'name': 'user/jdoe/5a/98/3_rse_remote_get.raw'}
-        :param domain: The network domain, either 'wan' (default) or 'lan'
-        :param auth_token: Optionally passing JSON Web Token (OIDC) string for authentication
-        :param logger: An optional logging.Logger object
+        :rse_settings:      RSE attributes
+        :param files:       a single dict or a list with dicts containing 'scope' and 'name'
+                            if LFNs are used and only 'name' if PFNs are used.
+                            E.g. {'name': '2_rse_remote_get.raw', 'scope': 'user.jdoe'}, {'name': 'user/jdoe/5a/98/3_rse_remote_get.raw'}
+        :param domain:      The network domain, either 'wan' (default) or 'lan'
+        :param auth_token:  Optionally passing JSON Web Token (OIDC) string for authentication
+        :param logger:      Optional decorated logger that can be passed from the calling daemons or servers.
 
-        :returns: True/False for a single file or a dict object with 'scope:name' for LFNs or 'name' for PFNs as keys and True or the exception as value for each file in bulk mode
+        :returns:           True/False for a single file or a dict object with 'scope:name' for LFNs or 'name' for PFNs as keys and True or the exception as value for each file in bulk mode
 
         :raises RSENotConnected: no connection to a specific storage has been established
     """
@@ -271,7 +274,7 @@ def exists(rse_settings, files, domain='wan', auth_token=None, logger=_logger):
     try:
         protocol.exists(None)
     except NotImplementedError:
-        protocol = create_protocol(rse_settings, 'write', domain=domain, auth_token=auth_token)
+        protocol = create_protocol(rse_settings, 'write', domain=domain, auth_token=auth_token, logger=logger)
         protocol.connect()
     except:
         pass
@@ -286,7 +289,7 @@ def exists(rse_settings, files, domain='wan', auth_token=None, logger=_logger):
             pfn = list(protocol.lfns2pfns(f).values())[0]
             if isinstance(pfn, exception.RucioException):
                 raise pfn
-            logger.debug('Checking if %s exists', pfn)
+            logger(logging.DEBUG, 'Checking if %s exists', pfn)
             # deal with URL signing if required
             if rse_settings['sign_url'] is not None and pfn[:5] == 'https':
                 pfn = __get_signed_url(rse_settings['rse'], rse_settings['sign_url'], 'read', pfn)    # NOQA pylint: disable=undefined-variable
@@ -305,29 +308,29 @@ def exists(rse_settings, files, domain='wan', auth_token=None, logger=_logger):
     return [gs, ret]
 
 
-def upload(rse_settings, lfns, domain='wan', source_dir=None, force_pfn=None, force_scheme=None, transfer_timeout=None, delete_existing=False, sign_service=None, auth_token=None, logger=_logger):
+def upload(rse_settings, lfns, domain='wan', source_dir=None, force_pfn=None, force_scheme=None, transfer_timeout=None, delete_existing=False, sign_service=None, auth_token=None, logger=logging.log):
     """
         Uploads a file to the connected storage.
         Providing a list indicates the bulk mode.
 
-        :rse_settings:   RSE attributes
-        :param lfns:        a single dict or a list with dicts containing 'scope' and 'name'.
-                            Examples:
-                            [
-                            {'name': '1_rse_local_put.raw', 'scope': 'user.jdoe', 'filesize': 42, 'adler32': '87HS3J968JSNWID'},
-                            {'name': '2_rse_local_put.raw', 'scope': 'user.jdoe', 'filesize': 4711, 'adler32': 'RSSMICETHMISBA837464F'}
-                            ]
-                            If the 'filename' key is present, it will be used by Rucio as the actual name of the file on disk (separate from the Rucio 'name').
-        :param domain: The network domain, either 'wan' (default) or 'lan'
-        :param source_dir:  path to the local directory including the source files
-        :param force_pfn: use the given PFN -- can lead to dark data, use sparingly
-        :param force_scheme: use the given protocol scheme, overriding the protocol priority in the RSE description
-        :param transfer_timeout: set this timeout (in seconds) for the transfers, for protocols that support it
-        :param sign_service: use the given service (e.g. gcs, s3, swift) to sign the URL
-        :param auth_token: Optionally passing JSON Web Token (OIDC) string for authentication
-        :param logger: An optional logging.Logger object
+        :rse_settings:            RSE attributes
+        :param lfns:              a single dict or a list with dicts containing 'scope' and 'name'.
+                                  Examples:
+                                  [
+                                  {'name': '1_rse_local_put.raw', 'scope': 'user.jdoe', 'filesize': 42, 'adler32': '87HS3J968JSNWID'},
+                                  {'name': '2_rse_local_put.raw', 'scope': 'user.jdoe', 'filesize': 4711, 'adler32': 'RSSMICETHMISBA837464F'}
+                                  ]
+                                  If the 'filename' key is present, it will be used by Rucio as the actual name of the file on disk (separate from the Rucio 'name').
+        :param domain:            The network domain, either 'wan' (default) or 'lan'
+        :param source_dir:        path to the local directory including the source files
+        :param force_pfn:         use the given PFN -- can lead to dark data, use sparingly
+        :param force_scheme:      use the given protocol scheme, overriding the protocol priority in the RSE description
+        :param transfer_timeout:  set this timeout (in seconds) for the transfers, for protocols that support it
+        :param sign_service:      use the given service (e.g. gcs, s3, swift) to sign the URL
+        :param auth_token:        Optionally passing JSON Web Token (OIDC) string for authentication
+        :param logger:            Optional decorated logger that can be passed from the calling daemons or servers.
 
-        :returns: True/False for a single file or a dict object with 'scope:name' as keys and True or the exception as value for each file in bulk mode
+        :returns:                 True/False for a single file or a dict object with 'scope:name' as keys and True or the exception as value for each file in bulk mode
 
         :raises RSENotConnected: no connection to a specific storage has been established
         :raises SourceNotFound: local source file can not be found
@@ -380,7 +383,7 @@ def upload(rse_settings, lfns, domain='wan', source_dir=None, force_pfn=None, fo
             else:
                 if protocol.exists('%s.rucio.upload' % pfn):  # Check for left over of previous unsuccessful attempts
                     try:
-                        logger.debug('Deleting %s.rucio.upload', pfn)
+                        logger(logging.DEBUG, 'Deleting %s.rucio.upload', pfn)
                         protocol_delete.delete('%s.rucio.upload' % list(protocol_delete.lfns2pfns(make_valid_did(lfn)).values())[0])
                     except Exception as e:
                         ret['%s:%s' % (scope, name)] = exception.RSEOperationNotSupported('Unable to remove temporary file %s.rucio.upload: %s' % (pfn, str(e)))
@@ -390,7 +393,7 @@ def upload(rse_settings, lfns, domain='wan', source_dir=None, force_pfn=None, fo
                 if delete_existing:
                     if protocol.exists('%s' % pfn):  # Check for previous completed uploads that have to be removed before upload
                         try:
-                            logger.debug('Deleting %s', pfn)
+                            logger(logging.DEBUG, 'Deleting %s', pfn)
                             protocol_delete.delete('%s' % list(protocol_delete.lfns2pfns(make_valid_did(lfn)).values())[0])
                         except Exception as e:
                             ret['%s:%s' % (scope, name)] = exception.RSEOperationNotSupported('Unable to remove file %s: %s' % (pfn, str(e)))
@@ -398,7 +401,7 @@ def upload(rse_settings, lfns, domain='wan', source_dir=None, force_pfn=None, fo
                             continue
 
                 try:  # Try uploading file
-                    logger.debug('Uploading to %s.rucio.upload', pfn)
+                    logger(logging.DEBUG, 'Uploading to %s.rucio.upload', pfn)
                     protocol.put(base_name, '%s.rucio.upload' % pfn, source_dir, transfer_timeout=transfer_timeout)
                 except Exception as e:
                     gs = False
@@ -436,7 +439,7 @@ def upload(rse_settings, lfns, domain='wan', source_dir=None, force_pfn=None, fo
 
                 if valid:  # The upload finished successful and the file can be renamed
                     try:
-                        logger.debug('Renaming %s.rucio.upload to %s', pfn, pfn)
+                        logger(logging.DEBUG, 'Renaming %s.rucio.upload to %s', pfn, pfn)
                         protocol.rename('%s.rucio.upload' % pfn, pfn)
                         ret['%s:%s' % (scope, name)] = True
                     except Exception as e:
@@ -453,7 +456,7 @@ def upload(rse_settings, lfns, domain='wan', source_dir=None, force_pfn=None, fo
                 gs = False
             else:
                 try:  # Try uploading file
-                    logger.debug('Uploading to %s', pfn)
+                    logger(logging.DEBUG, 'Uploading to %s', pfn)
                     protocol.put(base_name, pfn, source_dir, transfer_timeout=transfer_timeout)
                 except Exception as e:
                     gs = False
@@ -505,17 +508,17 @@ def upload(rse_settings, lfns, domain='wan', source_dir=None, force_pfn=None, fo
     return {0: gs, 1: ret, 'success': gs, 'pfn': pfn}
 
 
-def delete(rse_settings, lfns, domain='wan', auth_token=None):
+def delete(rse_settings, lfns, domain='wan', auth_token=None, logger=logging.log):
     """
         Delete a file from the connected storage.
         Providing a list indicates the bulk mode.
 
-        :rse_settings:   RSE attributes
-        :param lfns:        a single dict or a list with dicts containing 'scope' and 'name'. E.g. [{'name': '1_rse_remote_delete.raw', 'scope': 'user.jdoe'}, {'name': '2_rse_remote_delete.raw', 'scope': 'user.jdoe'}]
-        :param domain: The network domain, either 'wan' (default) or 'lan'
+        :rse_settings:     RSE attributes
+        :param lfns:       a single dict or a list with dicts containing 'scope' and 'name'. E.g. [{'name': '1_rse_remote_delete.raw', 'scope': 'user.jdoe'}, {'name': '2_rse_remote_delete.raw', 'scope': 'user.jdoe'}]
+        :param domain:     The network domain, either 'wan' (default) or 'lan'
         :param auth_token: Optionally passing JSON Web Token (OIDC) string for authentication
-
-        :returns: True/False for a single file or a dict object with 'scope:name' as keys and True or the exception as value for each file in bulk mode
+        :param logger:     Optional decorated logger that can be passed from the calling daemons or servers.
+        :returns:          True/False for a single file or a dict object with 'scope:name' as keys and True or the exception as value for each file in bulk mode
 
         :raises RSENotConnected: no connection to a specific storage has been established
         :raises SourceNotFound: remote source file can not be found on storage
@@ -525,7 +528,7 @@ def delete(rse_settings, lfns, domain='wan', auth_token=None):
     ret = {}
     gs = True  # gs represents the global status which inidcates if every operation workd in bulk mode
 
-    protocol = create_protocol(rse_settings, 'delete', domain=domain, auth_token=auth_token)
+    protocol = create_protocol(rse_settings, 'delete', domain=domain, auth_token=auth_token, logger=logger)
     protocol.connect()
 
     lfns = [lfns] if not type(lfns) is list else lfns
@@ -548,24 +551,25 @@ def delete(rse_settings, lfns, domain='wan', auth_token=None):
     return [gs, ret]
 
 
-def rename(rse_settings, files, domain='wan', auth_token=None):
+def rename(rse_settings, files, domain='wan', auth_token=None, logger=logging.log):
     """
         Rename files stored on the connected storage.
         Providing a list indicates the bulk mode.
 
-        :rse_settings:   RSE attributes
-        :param files: a single dict or a list with dicts containing 'scope', 'name', 'new_scope' and 'new_name'
-                      if LFNs are used or only 'name' and 'new_name' if PFNs are used.
-                      If 'new_scope' or 'new_name' are not provided, the current one is used.
-                      Examples:
-                      [
-                      {'name': '3_rse_remote_rename.raw', 'scope': 'user.jdoe', 'new_name': '3_rse_new.raw', 'new_scope': 'user.jdoe'},
-                      {'name': 'user/jdoe/d9/cb/9_rse_remote_rename.raw', 'new_name': 'user/jdoe/c6/4a/9_rse_new.raw'}
-                      ]
-        :param domain: The network domain, either 'wan' (default) or 'lan'
+        :rse_settings:     RSE attributes
+        :param files:      a single dict or a list with dicts containing 'scope', 'name', 'new_scope' and 'new_name'
+                           if LFNs are used or only 'name' and 'new_name' if PFNs are used.
+                           If 'new_scope' or 'new_name' are not provided, the current one is used.
+                           Examples:
+                           [
+                           {'name': '3_rse_remote_rename.raw', 'scope': 'user.jdoe', 'new_name': '3_rse_new.raw', 'new_scope': 'user.jdoe'},
+                           {'name': 'user/jdoe/d9/cb/9_rse_remote_rename.raw', 'new_name': 'user/jdoe/c6/4a/9_rse_new.raw'}
+                           ]
+        :param domain:     The network domain, either 'wan' (default) or 'lan'
         :param auth_token: Optionally passing JSON Web Token (OIDC) string for authentication
+        :param logger:     Optional decorated logger that can be passed from the calling daemons or servers.
 
-        :returns: True/False for a single file or a dict object with LFN (key) and True/False (value) in bulk mode
+        :returns:          True/False for a single file or a dict object with LFN (key) and True/False (value) in bulk mode
 
         :raises RSENotConnected: no connection to a specific storage has been established
         :raises SourceNotFound: remote source file can not be found on storage
@@ -575,7 +579,7 @@ def rename(rse_settings, files, domain='wan', auth_token=None):
     ret = {}
     gs = True  # gs represents the global status which inidcates if every operation workd in bulk mode
 
-    protocol = create_protocol(rse_settings, 'write', domain=domain, auth_token=auth_token)
+    protocol = create_protocol(rse_settings, 'write', domain=domain, auth_token=auth_token, logger=logger)
     protocol.connect()
 
     files = [files] if not type(files) is list else files
@@ -623,23 +627,24 @@ def rename(rse_settings, files, domain='wan', auth_token=None):
     return [gs, ret]
 
 
-def get_space_usage(rse_settings, scheme=None, domain='wan', auth_token=None):
+def get_space_usage(rse_settings, scheme=None, domain='wan', auth_token=None, logger=logging.log):
     """
         Get RSE space usage information.
 
-        :rse_settings:   RSE attributes
-        :param scheme: optional filter to select which protocol to be used.
-        :param domain: The network domain, either 'wan' (default) or 'lan'
+        :rse_settings:     RSE attributes
+        :param scheme:     optional filter to select which protocol to be used.
+        :param domain:     The network domain, either 'wan' (default) or 'lan'
         :param auth_token: Optionally passing JSON Web Token (OIDC) string for authentication
+        :param logger:     Optional decorated logger that can be passed from the calling daemons or servers.
 
-        :returns: a list with dict containing 'totalsize' and 'unusedsize'
+        :returns:          a list with dict containing 'totalsize' and 'unusedsize'
 
         :raises ServiceUnavailable: if some generic error occured in the library.
     """
     gs = True
     ret = {}
 
-    protocol = create_protocol(rse_settings, 'read', scheme=scheme, domain=domain, auth_token=auth_token)
+    protocol = create_protocol(rse_settings, 'read', scheme=scheme, domain=domain, auth_token=auth_token, logger=logger)
     protocol.connect()
 
     try:
